local MakePlayerCharacter = require "prefabs/player_common"

local assets = {

    Asset( "IMAGE", "images/saveslot_portraits/wharang.tex" ),
    Asset( "ATLAS", "images/saveslot_portraits/wharang.xml" ),
    Asset( "IMAGE", "images/selectscreen_portraits/wharang.tex" ),
    Asset( "ATLAS", "images/selectscreen_portraits/wharang.xml" ),
    Asset( "IMAGE", "images/selectscreen_portraits/wharang_silho.tex" ),
    Asset( "ATLAS", "images/selectscreen_portraits/wharang_silho.xml" ),
    Asset( "IMAGE", "bigportraits/wharang.tex" ),
    Asset( "ATLAS", "bigportraits/wharang.xml" ),
	Asset( "IMAGE", "images/minimapicons/wharang.tex" ),
	Asset( "ATLAS", "images/minimapicons/wharang.xml" ),
	Asset( "IMAGE", "images/minimapicons/wharang_evil.tex" ),
	Asset( "ATLAS", "images/minimapicons/wharang_evil.xml" ),
	
	Asset( "ANIM", "anim/wharang.zip" ),
	Asset( "ANIM", "anim/wharangW.zip" ),	
}

local prefabs =
{
	 "wharang_spirit",
	 "wharang_spirit_evil",
}

local start_inv = 
{
	 "mihobell",
	 "handfan",
	 "sakurahat",
}

local function Wharang_Collection(inst)
	local max_soulheart = TUNING.SOULHEART_MAX
	local collect_soul = math.min(inst.soulheart, max_soulheart)
	
	local max_soulspirit = TUNING.SOULSPIRIT_MAX
	local collect_spirit = math.min(inst.soulspirit, max_soulspirit)
	
	-----------------------------------------------------------------------

	if inst.soulheart < max_soulheart then
		while inst.soulspirit >= max_soulspirit do
		
			inst.soulspirit = inst.soulspirit - max_soulspirit
			inst.soulheart = inst.soulheart + 1
				if inst.soulheart > max_soulheart then
					inst.soulheart = max_soulheart
				end
			
			inst.SoundEmitter:PlaySound("millenniumfox/characters/wharang/soulheart")
			
			local fx1 = SpawnPrefab("wharang_spiritfx")
			fx1.Transform:SetPosition(inst.Transform:GetWorldPosition());fx1.Transform:SetScale(.8,.8,.8)
			local fx2 = SpawnPrefab("wharang_multifireworkfx")
			fx2.Transform:SetPosition(inst.Transform:GetWorldPosition());fx2.Transform:SetScale(1.3,1.3,1.3)
			local fx3 = SpawnPrefab("sparklefx")
			fx3.Transform:SetPosition(inst.Transform:GetWorldPosition());fx2.Transform:SetScale(1.5,1.5,1.5)
			
			inst.HUD.controls.status.heart:PulseGreen()
			inst.HUD.controls.status.stomach:PulseGreen()
			inst.HUD.controls.status.brain:PulseGreen()
			
			inst.HUD.controls.status.brain:ScaleTo(1.3,1,.7)
			inst.HUD.controls.status.heart:ScaleTo(1.3,1,.7)
			inst.HUD.controls.status.stomach:ScaleTo(1.3,1,.7)
			
			inst.components.talker:Say((GetString(inst.prefab, "DESC_SOULCOLLECT")).." "..(inst.soulheart).." "..(GetString(inst.prefab, "DESC_SOULHEART")))
		end
	else
		inst.soulheart = max_soulheart
		inst.soulspirit = max_soulspirit
	end
	
	-----------------------------------------------------------------------

	local hunger_percent = inst.components.hunger:GetPercent()
	local health_percent = inst.components.health:GetPercent()
	local sanity_percent = inst.components.sanity:GetPercent()

	inst.components.health.maxhealth = math.ceil (TUNING.WHARANG_HEALTH_S + collect_soul * TUNING.WHARANG_HEALTH_PLUS) --280
	inst.components.hunger.max = math.ceil (TUNING.WHARANG_HUNGER_S + collect_soul * TUNING.WHARANG_HUNGER_PLUS) --185
	inst.components.sanity.max = math.ceil (TUNING.WHARANG_SANITY_S + collect_soul * TUNING.WHARANG_SANITY_PLUS) --400

    inst.components.hunger:SetPercent(hunger_percent)
    inst.components.health:SetPercent(health_percent)
    inst.components.sanity:SetPercent(sanity_percent)

end

local function Get_SoulHeart(inst)
	inst.soulspirit = inst.soulspirit + 1000
	Wharang_Collection(inst)
end

local function Get_SoulHeart_10(inst)
	inst.soulspirit = inst.soulspirit + 10000
	Wharang_Collection(inst)
end

local function Get_SoulHeart_100(inst)
	inst.soulspirit = inst.soulspirit + 100000
	Wharang_Collection(inst)
end



local function Wharang_SanityFire(inst)
	local x,y,z = inst.Transform:GetWorldPosition()	
	local delta = 0
	local max_rad = 10
	local ents = TheSim:FindEntities(x,y,z, max_rad, {"fire"})
    for k,v in pairs(ents) do 
    	if v.components.burnable and v.components.burnable.burning then
    		local sz = TUNING.SANITYAURA_TINY
    		local rad = v.components.burnable:GetLargestLightRadius() or 1
    		sz = sz * ( math.min(max_rad, rad) / max_rad )
			local distsq = inst:GetDistanceSqToInst(v)
			delta = delta + sz/math.max(1, distsq)
    	end
    end
    
    return delta
end

local function AuraTarget(target)
	if (target:HasTag("monster") and not target:HasTag("shadowcreature")) then
		return true
	end
	return false
end

local function Wharang_EvilAura(inst, dt)
	if inst.strength == "evilwhiteform" or inst.strength == "whiteform" then
    local AuraRange = 2.5
	local hits = inst.components.combat:DoAreaAttack(inst, AuraRange, inst, AuraTarget) end
end

local function OnKill_FX(inst, data)
	if data.inst:HasTag("smallcreature") and not data.inst:HasTag("monster") then
		local fx = SpawnPrefab("wharang_spiritfx")
		fx.Transform:SetPosition(data.inst:GetPosition():Get());fx.Transform:SetScale(.2,.2,.2)
	elseif data.inst:HasTag("largecreature") and not data.inst:HasTag("monster") then
		local fx = SpawnPrefab("wharang_spiritfx")
		fx.Transform:SetPosition(data.inst:GetPosition():Get());fx.Transform:SetScale(.6,.6,.6)
	elseif data.inst:HasTag("monster") then
		local fx = SpawnPrefab("wharang_spirit_evilfx")
		fx.Transform:SetPosition(data.inst:GetPosition():Get());fx.Transform:SetScale(.4,.4,.4)
	else
		local fx = SpawnPrefab("wharang_spiritfx")
		fx.Transform:SetPosition(data.inst:GetPosition():Get());fx.Transform:SetScale(.4,.4,.4)
	end
end

local function IsValidVictim(victim)
	return victim ~= nil
		and not ((victim:HasTag("prey") and not victim:HasTag("hostile")) or
				victim:HasTag("player") or
				victim:HasTag("companion") or
				victim:HasTag("wall") or
				victim:HasTag("structure") or
				victim:HasTag("ghost") or
				victim:HasTag("veggie") or
				victim:HasTag("birchnutdrake") or
				victim.prefab == ("tentacle_pillar_arm"))
		and victim.components.health ~= nil
		and victim.components.combat ~= nil
end

local function Wharang_OnKill(inst, data)
	local time = data.inst.components.health.destroytime or 2
	local victim = data.inst

	if IsValidVictim(victim) and data.cause == inst.prefab then
		if inst.soulheart < TUNING.SOULHEART_MAX then
			if data.inst:HasTag("epic") then
				inst.soulspirit = inst.soulspirit + 500
			elseif data.inst:HasTag("nightmarecreature") then
				inst.soulspirit = inst.soulspirit + math.random( 20, 50 )
			elseif data.inst:HasTag("worm") or data.inst:HasTag("WORM_DANGER") then
				inst.soulspirit = inst.soulspirit + math.random( 15, 30 )
			elseif data.inst:HasTag("shadowchesspiece") or data.inst:HasTag("tentacle_pillar") then
				inst.soulspirit = inst.soulspirit + math.random( 20, 40 )
			elseif data.inst:HasTag("largecreature") or data.inst:HasTag("mossling") then
				inst.soulspirit = inst.soulspirit + math.random( 10, 20 )
			elseif data.inst:HasTag("monster") or data.inst:HasTag("character") then
				inst.soulspirit = inst.soulspirit + math.random( 5, 10 )
			else
				inst.soulspirit = inst.soulspirit + math.random( 1, 5 )
			end
			inst:DoTaskInTime(time, function() OnKill_FX(inst, data) end)
			
			--inst.components.talker:Say("("..(math.floor(inst.soulspirit/10)).."% "..(GetString(inst.prefab, "DESC_SOULCOLLECT_SPIRIT"))..")")
			Wharang_Collection(inst)
		end
	end

end

local function Wharang_OnEat(inst, food)

	local prank_1 = 3/100
	local prank_2 = 5/100
	local prank_3 = 4/100
	local prank_4 = 2/100
	local prank_5 = 1/100

	if food and food.components.edible.foodtype == "MEAT" then
		if inst.soulheart >= 100 then if math.random() < prank_1 then
			local pos = Vector3(inst.Transform:GetWorldPosition());pos.z = pos.z + math.random();pos.x = pos.x + math.random()
			local item = {"crow","feather_crow","spider","fish","froglegs","rabbit","bee","eel","butterfly","pigskin","tentaclespots","stinger"}
			local randomitem = item[math.random(#item)]
			SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition());SpawnPrefab(randomitem).Transform:SetPosition(pos:Get()) end
		end
		if inst.soulheart >= 200 then if math.random() < prank_2 then
			local pos = Vector3(inst.Transform:GetWorldPosition());pos.z = pos.z + math.random();pos.x = pos.x + math.random()
			local item = {"rope","cutstone","boards","papyrus","charcoal","ash","twigs","rocks","cutgrass","cutreeds","log","nitre"}
			local randomitem = item[math.random(#item)]
			SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition());SpawnPrefab(randomitem).Transform:SetPosition(pos:Get()) end
		end
		if inst.soulheart >= 300 then if math.random() < prank_3 then
			local pos = Vector3(inst.Transform:GetWorldPosition());pos.z = pos.z + math.random();pos.x = pos.x + math.random()
			local item = {"silk","beefalowool","manrabbit_tail","beardhair","slurper_pelt","slurtleslime","honeycomb"}
			local randomitem = item[math.random(#item)]
			SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition());SpawnPrefab(randomitem).Transform:SetPosition(pos:Get()) end
		end
		if inst.soulheart >= 400 then if math.random() < prank_4 then
			local pos = Vector3(inst.Transform:GetWorldPosition());pos.z = pos.z + math.random();pos.x = pos.x + math.random()
			local item = {"spoiled_food","wetgoop","powcake","monsterlasagna","meatballs","honeyham","taffy","unagi","honey","cutlichen"}
			local randomitem = item[math.random(#item)]
			SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition());SpawnPrefab(randomitem).Transform:SetPosition(pos:Get()) end
		end
		if inst.soulheart >= 500 then if math.random() < prank_5 then
			local pos = Vector3(inst.Transform:GetWorldPosition());pos.z = pos.z + math.random();pos.x = pos.x + math.random()
			local item = {"gears","petals_evil","foliage","nightmarefuel","goldnugget","thulecite","thulecite_pieces","redgem","bluegem","yellowgem"}
			local randomitem = item[math.random(#item)]
			SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition());SpawnPrefab(randomitem).Transform:SetPosition(pos:Get()) end
		end
	end
end

-------------------------------------------Party
local function Show_Party(inst, dt)
	if inst:HasTag("FireFoxAttack") then
		if not inst:HasTag("PartyEnd") then
			inst.party_time = inst.party_time - dt
			if inst.party_time <= 0 then
				inst.party_time = .35
				local nightmare = SpawnPrefab("wharang_ground_firefx")
				nightmare.Transform:SetPosition(inst.Transform:GetWorldPosition())
				nightmare.Transform:SetScale(.25,.25,.25)
	    	end
	    end
	end
end

local function PartyEnd(inst)
	if inst.PartyTime > 0 then
		inst.PartyTime = inst.PartyTime-1
	else
		inst:RemoveTag("PartyEnd")
		inst.components.talker:Say(GetString(inst.prefab,"ANNOUNCE_CAN_PARTY"))
		if inst.party_task then inst.party_task:Cancel() inst.party_task = nil end
	end
end

local function Start_Fire_Party(inst, data)
local other = data.target
if other
and not other:HasTag("veggie")
and not other:HasTag("structure")
and not other:HasTag("companion") then

	if inst:HasTag("FireFoxAttack") then
		if not inst:HasTag("PartyEnd") then
			 inst.components.sanity:DoDelta(TUNING.SOULHEART_FIREWORK_DELTA_SAN)
			 inst.components.playercontroller:ShakeCamera(inst, "FULL", 0.5, 0.02, 1.5, 30)
			 inst.components.groundpounderds.groundpoundfx = "wharang_ground_firefx"
			 inst.components.groundpounderds.groundpoundringfx = "wharang_ringfx"
			if (100 <= inst.soulheart) and (inst.soulheart < 200) then
			 inst.components.groundpounderds.numRings = TUNING.SOULHEART_FIREWORK_RANK_1_RING
			 inst.components.groundpounderds.damageRings = TUNING.SOULHEART_FIREWORK_RANK_1_RING_DAM
			 inst.components.groundpounderds.groundpounddamagemult = TUNING.SOULHEART_FIREWORK_RANK_1_MULT_DAM
			 inst.PartyTime = TUNING.SOULHEART_FIREWORK_RANK_1_TIME
			elseif (200 <= inst.soulheart) and (inst.soulheart < 300) then
			 inst.components.groundpounderds.numRings = TUNING.SOULHEART_FIREWORK_RANK_2_RING
			 inst.components.groundpounderds.damageRings = TUNING.SOULHEART_FIREWORK_RANK_2_RING_DAM
			 inst.components.groundpounderds.groundpounddamagemult = TUNING.SOULHEART_FIREWORK_RANK_2_MULT_DAM
			 inst.Bonus_Party_1 = inst:DoPeriodicTask(1, function() inst.components.groundpounderds:GroundPound() if inst.Bonus_Party_1 then inst.Bonus_Party_1:Cancel() inst.Bonus_Party_1 = nil end end)
			 inst.PartyTime = TUNING.SOULHEART_FIREWORK_RANK_2_TIME
			elseif (300 <= inst.soulheart) and (inst.soulheart < 400) then
			 inst.components.groundpounderds.numRings = TUNING.SOULHEART_FIREWORK_RANK_3_RING
			 inst.components.groundpounderds.damageRings = TUNING.SOULHEART_FIREWORK_RANK_3_RING_DAM
			 inst.components.groundpounderds.groundpounddamagemult = TUNING.SOULHEART_FIREWORK_RANK_3_MULT_DAM
			 inst.Bonus_Party_2 = inst:DoPeriodicTask(1, function() inst.components.groundpounderds:GroundPound() if inst.Bonus_Party_2 then inst.Bonus_Party_2:Cancel() inst.Bonus_Party_2 = nil end end)
			 inst.PartyTime = TUNING.SOULHEART_FIREWORK_RANK_3_TIME
			elseif (400 <= inst.soulheart) then
			 inst.components.groundpounderds.numRings = TUNING.SOULHEART_FIREWORK_RANK_4_RING
			 inst.components.groundpounderds.damageRings = TUNING.SOULHEART_FIREWORK_RANK_4_RING_DAM
			 inst.components.groundpounderds.groundpounddamagemult = TUNING.SOULHEART_FIREWORK_RANK_4_MULT_DAM
			 inst.Bonus_Party_3 = inst:DoPeriodicTask(.5, function() inst.components.groundpounderds:GroundPound() if inst.Bonus_Party_3 then inst.Bonus_Party_3:Cancel() inst.Bonus_Party_3 = nil end end)
			 inst.Bonus_Party_4 = inst:DoPeriodicTask(1, function() inst.components.groundpounderds:GroundPound() if inst.Bonus_Party_4 then inst.Bonus_Party_4:Cancel() inst.Bonus_Party_4 = nil end end)
			 inst.PartyTime = TUNING.SOULHEART_FIREWORK_RANK_4_TIME
			end
			 inst.components.groundpounderds:GroundPound()
			 inst:AddTag("PartyEnd");inst:RemoveTag("FireFoxAttack")
			 inst.party_task = inst:DoPeriodicTask(1/10, function() PartyEnd(inst) end)
		elseif inst:HasTag("PartyEnd") then
		end
	end
end
end

local function TheEvilWhite(inst, dt)
    if inst.strength == "evilwhiteform" or inst.strength == "whiteform" then
    	inst.nightmare_time = inst.nightmare_time - dt
    	if inst.nightmare_time <= 0 then
    		inst.nightmare_time = 1
			local pos = Vector3(inst.Transform:GetWorldPosition())
			pos.y = pos.y + .5
			local nightmare = SpawnPrefab("statue_transition_2")
			nightmare.Transform:SetPosition(pos:Get())
    	end
    end
end

local function FormChange_FX(inst)
	SpawnPrefab("statue_transition").Transform:SetPosition(inst:GetPosition():Get())
	SpawnPrefab("statue_transition_2").Transform:SetPosition(inst:GetPosition():Get())
end

local function Wharang_OnStatus(inst)
	if GetClock():IsNight() and GetClock():GetMoonPhase() == "full" then
		if inst.strength ~= "whiteform" then
			inst.strength = "whiteform"
			FormChange_FX(inst)
		end
	elseif inst.components.sanity:GetPercent() < .2 then
		if inst.strength ~= "evilwhiteform" then
			inst.strength = "evilwhiteform"
			FormChange_FX(inst)
		end
	elseif inst.strength ~= "normalform" then
		inst.strength = "normalform"
		FormChange_FX(inst)
	end
end

local function Wharang_Stats(inst)
	local max_soulheart = TUNING.SOULHEART_MAX
	local collect_soul = math.min(inst.soulheart, max_soulheart)


	Wharang_OnStatus(inst)

	if inst.strength == "whiteform" then
	
		inst.AnimState:SetBuild("wharangW")
		
		inst.soundsname = "wharang_evilwhite"
		inst.components.eater:SetOmnivore()
		inst.components.talker.colour = Vector3(54/255, 198/255, 250/255)
		
		inst.components.combat.damagemultiplier = 1.0
		inst.components.temperature.inherentinsulation = math.huge
		
		inst.components.locomotor.walkspeed = math.ceil (TUNING.WHARANG_WALKSPEED + 2 + inst.soulheart * 0.002)
		inst.components.locomotor.runspeed = math.ceil (TUNING.WHARANG_RUNSPEED + 2 + inst.soulheart * 0.002)
		
	elseif inst.strength == "evilwhiteform" then
	
		inst.AnimState:SetBuild("wharangW")
		
		inst.soundsname = "wharang_evilwhite"
		inst.components.eater:SetCarnivore()
		inst.components.talker.colour = Vector3(54/255, 198/255, 250/255)
		
		inst.components.combat.damagemultiplier = 1.0
		inst.components.temperature.inherentinsulation = 30

		inst.components.locomotor.walkspeed = math.ceil (TUNING.WHARANG_WALKSPEED + 2 + inst.soulheart * 0.002)
		inst.components.locomotor.runspeed = math.ceil (TUNING.WHARANG_RUNSPEED + 2 + inst.soulheart * 0.002)
		
	elseif inst.strength == "normalform" then
	
		inst.AnimState:SetBuild("wharang")
		
		inst.soundsname = "winnie"
		inst.components.eater:SetOmnivore()
		inst.components.talker.colour = Vector3(255/255, 165/255, 154/255)
		
		inst.components.combat.damagemultiplier = 0.8
		inst.components.temperature.inherentinsulation = 30
		
		inst.components.locomotor.walkspeed = math.ceil (TUNING.WHARANG_WALKSPEED + inst.soulheart * 0.002)
		inst.components.locomotor.runspeed = math.ceil (TUNING.WHARANG_RUNSPEED + inst.soulheart * 0.002)
	end
	
end

local function Wharang_FlameBody(inst, data)
	if data.attacker and data.attacker.components.burnable and data.attacker.components.health and not data.attacker:HasTag("thorny") then
		data.attacker.components.health:DoDelta(-2)
		data.attacker.components.burnable:Ignite()
		SpawnPrefab("foxfire").Transform:SetPosition(data.attacker.Transform:GetWorldPosition())
	end
	
	if data.attacker and data.attacker.components.burnable and data.attacker.components.burnable:IsBurning() then
		data.attacker.components.burnable:Extinguish()
	end
end

local function Wharang_NightVision(inst, data)
local Light = inst.entity:AddLight()

	if GetClock():IsNight() or GetWorld():IsCave() then
		if inst.strength == "normalform" then
			if inst.components.hunger:GetPercent() > TUNING.WHARANG_LIGHT_HUNGER then
				inst.Light:SetRadius(1.1)
				inst.Light:SetFalloff(.7)
				inst.Light:SetIntensity(.5)
				inst.Light:SetColour(255/255,20/255,20/255)
				inst.Light:Enable(true)
			else
				Light:SetRadius(0);Light:SetFalloff(0);Light:SetIntensity(0);Light:SetColour(0,0,0)
				Light:Enable(false)
			end

		else
			inst.Light:SetRadius(1.1)
			inst.Light:SetFalloff(.7)
			inst.Light:SetIntensity(.5)
			inst.Light:SetColour(20/255,255/255,255/255)
			inst.Light:Enable(true)
		end
		
	else
		Light:SetRadius(0);Light:SetFalloff(0);Light:SetIntensity(0);Light:SetColour(0,0,0)
		Light:Enable(false)
	end
end

local function Wharang_Regen(inst)
    inst.components.health:DoDelta(1)
end

-------------------------------------------

local function onpreload(inst, data)
	if data then
		if data.soulheart then inst.soulheart = data.soulheart
			Wharang_Collection(inst)
		end
		if data.soulspirit then inst.soulspirit = data.soulspirit
			Wharang_Collection(inst)
		end
		
		if data.health and data.health.health then
			inst.components.health.currenthealth = data.health.health
		end
		if data.hunger and data.hunger.hunger then
			inst.components.hunger.current = data.hunger.hunger
		end
		if data.sanity and data.sanity.current then
			inst.components.sanity.current = data.sanity.current
		end
			
		inst.components.health:DoDelta(0)
		inst.components.hunger:DoDelta(0)
		inst.components.sanity:DoDelta(0)
	end
	
	if data.timeout then
        inst.PartyTime = data.timeout
		if inst.PartyTime > 0 then
			if not inst.party_task then
				inst.party_task = inst:DoPeriodicTask(1, function() PartyEnd(inst) end)
				inst:AddTag("PartyEnd")
			end
		else
			if inst.party_task then inst.party_task:Cancel() inst.party_task = nil end
			inst:RemoveTag("PartyEnd")
		end
    end
end

local function onsave(inst, data)
	data.soulheart = inst.soulheart
	data.soulspirit = inst.soulspirit
	data.timeout = (inst.PartyTime)
end

-------------------------------------------

local fn = function(inst)
	
	inst.soundsname = "winnie"
	inst.MiniMapEntity:SetIcon("wharang.tex")
	
	inst.strength = "normalform"
	
	inst.soulheart = 0
	inst.soulspirit = 0
	Wharang_Collection(inst)
	
	table.insert(inst.components.eater.foodprefs, "MEAT")
	
	inst:AddComponent("eater")
    inst.components.eater:SetCanEatHorrible() 
    inst.components.eater:SetOnEatFn(Wharang_OnEat)
	inst.components.eater.WharangMEAT = inst.components.eater.Eat
	function inst.components.eater:Eat( food )
		if self:CanEat(food) then
			if food.prefab == "meat" or food.prefab == "smallmeat" or food.prefab == "batwing" or food.prefab == "froglegs" or food.prefab == "plantmeat" then
					food.components.edible.sanityvalue = 0
			elseif food.prefab == "monstermeat" then
					food.components.edible.healthvalue = 0
					food.components.edible.sanityvalue = -30
			elseif food.prefab == "cookedmonstermeat" then
					food.components.edible.healthvalue = 0
					food.components.edible.sanityvalue = -20
			elseif food.prefab == "monstermeat_dried" then
					food.components.edible.healthvalue = 0
					food.components.edible.sanityvalue = -15
			elseif food.prefab == "drumstick" then
					food.components.edible.healthvalue = 2
					food.components.edible.hungervalue = 15
					food.components.edible.sanityvalue = 5
			elseif food.prefab == "drumstick_cooked" then
					food.components.edible.healthvalue = 5
					food.components.edible.hungervalue = 18
					food.components.edible.sanityvalue = 5
			elseif food.prefab == "turkeydinner" then
					food.components.edible.healthvalue = 40
					food.components.edible.sanityvalue = 10
			elseif food.prefab == "monsterlasagna" then
					food.components.edible.healthvalue = 0
					food.components.edible.sanityvalue = -40
			end
		end
	return inst.components.eater:WharangMEAT(food)
	end
	
	-- stats
	inst.components.health:SetMaxHealth(TUNING.WHARANG_HEALTH_S)
	inst.components.hunger:SetMax(TUNING.WHARANG_HUNGER_S)
	inst.components.sanity:SetMax(TUNING.WHARANG_SANITY_S)
	
	-- speed
	inst.components.locomotor.walkspeed = (TUNING.WHARANG_WALKSPEED)
	inst.components.locomotor.runspeed = (TUNING.WHARANG_RUNSPEED)
	
	-- +
	inst.components.health.fire_damage_scale = 0
	inst.components.sanity.night_drain_mult = 0
	inst.components.sanity.neg_aura_mult = 2
	inst.components.sanity.custom_rate_fn = Wharang_SanityFire
	
	inst.components.builder.magic_bonus = 2
	
	-- tag
	inst:AddTag("millenniumfox")
	inst:AddTag("wharang")

	-- on kill
	inst:ListenForEvent("entity_death", function(wrld, data) Wharang_OnKill(inst, data) end, GetWorld())
	
	-- health regen
	inst:DoPeriodicTask(60, Wharang_Regen)
	
	-- status
	inst:ListenForEvent("sanitydelta", Wharang_Stats)
    inst:ListenForEvent("daytime", function() Wharang_Stats(inst) end, GetWorld())
	inst:ListenForEvent("dusktime", function() Wharang_Stats(inst) end, GetWorld())
    inst:ListenForEvent("nighttime", function() Wharang_Stats(inst) end, GetWorld())
	inst:DoPeriodicTask(0, Wharang_Stats)
    Wharang_Stats(inst)
	
	-- attacked
	inst:ListenForEvent("attacked", Wharang_FlameBody)
	
	-- night vision
	inst:ListenForEvent("hungerdelta", Wharang_NightVision)
	inst:ListenForEvent("daytime", function() Wharang_NightVision(inst) end, GetWorld())
	inst:ListenForEvent("nighttime", function() Wharang_NightVision(inst) end, GetWorld())
	
	-- skill
	inst:AddComponent("groundpounderds")
	inst:ListenForEvent("onhitother", Start_Fire_Party)
	
	inst.party_time = .35
	inst:DoPeriodicTask(1/10, function() Show_Party(inst, 1/10) end) --FX

	-- evil aura
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(4)
    inst.updateTask = inst:DoPeriodicTask(1, Wharang_EvilAura, nil, 1)
	
	inst.nightmare_time = 1
	inst:DoPeriodicTask(1/10, function() TheEvilWhite(inst, 1/10) end) --FX

	-- recipe
	RECIPETABS['YINYANG'] = {str = "YINYANG", sort= 99, icon = "yinyangtab.tex", icon_atlas = "images/tabs/yinyangtab.xml"}
	
	-- DLCs
	if IsROG and not IsSW then
		inst.components.temperature.overheattemp = 80
		
	elseif IsSW then
		inst.components.temperature.overheattemp = 80
		inst.components.poisonable.duration = 80
		--[[
		inst:DoPeriodicTask(80,function()
			if inst.components.poisonable then
				inst.components.poisonable:Cure(inst)
			end
		end)
		]]
	end
	
	inst.OnSave = onsave
	inst.OnPreLoad = onpreload
	
	-- for console
	-- GetPlayer():PushEvent("soulheartup")
	inst:ListenForEvent("soulheartup", Get_SoulHeart)
	inst:ListenForEvent("soulheartup10", Get_SoulHeart_10)
	inst:ListenForEvent("soulheartup100", Get_SoulHeart_100)
	
end

return MakePlayerCharacter("wharang", prefabs, assets, fn, start_inv)