require "prefabutil"

local Assets =
{
	Asset("ANIM", "anim/wharang_dango.zip"),
	Asset("ATLAS", "images/inventoryimages/wharang_dango.xml"),
}

local function Dango_OnEaten(inst, eater)
	if GetClock():IsNight() and GetClock():GetMoonPhase() == "full" then
		inst.SoundEmitter:PlaySound("dontstarve/rabbit/scream_short")
	end
	
	if eater.dango then
		eater.dango.components.spell.lifetime = 0
		eater.dango.components.spell:ResumeSpell()
	else
		local light = SpawnPrefab("dango_light")
        
		light.components.spell:SetTarget(eater)
		if not light.components.spell.target then
			light:Remove() end
		light.components.spell:StartSpell()
    end
end

local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("wharang_dango")
	inst.AnimState:SetBuild("wharang_dango")
	inst.AnimState:PlayAnimation("idle", true)

	if IsSW == true then
		MakeInventoryFloatable(inst, "idle_water", "idle")
		MakeBlowInHurricane(inst,TUNING.WINDBLOWN_SCALE_MIN.LIGHT,TUNING.WINDBLOWN_SCALE_MAX.LIGHT)
	end

	inst:AddTag("wharangmod")
	inst:AddTag("dango")
	inst:AddTag("cattoy")
	
	inst:AddComponent("inspectable")
	inst:AddComponent("tradable")
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "GENERIC"
	inst.components.edible.hungervalue = (25)
	inst.components.edible.healthvalue = (10)
	inst.components.edible.sanityvalue = (5)
	inst.components.edible:SetOnEatenFn(Dango_OnEaten)
	
	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.PERISH_MED)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "petals"
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/inventoryimages/wharang_dango.xml"
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
	
	return inst
end

local function light_resume(inst, time)
	local percent = time/inst.components.spell.duration
	local var = inst.components.spell.variables
	if percent and time > 0 then
		inst.components.lighttweener:StartTween(inst.light, Lerp(0, var.radius, percent), 0.8, 0.5, {1,1,1}, 0)
		inst.components.lighttweener:StartTween(nil, 0, nil, nil, nil, time)
	end
end

local function light_onsave(inst, data)
	data.timealive = inst:GetTimeAlive()
end

local function light_onload(inst, data)
	if data and data.timealive then
		light_resume(inst, data.timealive)
	end
end

local function light_spellfn(inst, target, variables)
	if target then
		inst.Transform:SetPosition(target:GetPosition():Get())
	end
end

local function light_start(inst)
	local spell = inst.components.spell
	inst.components.lighttweener:StartTween(inst.light, spell.variables.radius, 0.8, 0.5, {169/255,231/255,245/255}, 0)
	inst.components.lighttweener:StartTween(nil, 0, nil, nil, nil, spell.duration)
end

local function light_ontarget(inst, target)
	if not target then return end
	target.dango = inst
	target:AddTag(inst.components.spell.spellname)
	target.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
end

local function light_onfinish(inst)
	if not inst.components.spell.target then return end
	inst.components.spell.target.dango = nil
	inst.components.spell.target.AnimState:ClearBloomEffectHandle()
end

local light_variables = { radius = TUNING.DANGO_RADIUS }

local function lightfn()

	local inst = CreateEntity()
	inst.entity:AddTransform()
	
	inst:AddComponent("lighttweener")
	
	inst.light = inst.entity:AddLight()
	inst.light:Enable(true)
	
	inst:AddTag("FX")
	inst:AddTag("NOCLICK")
	
	local spell = inst:AddComponent("spell")
	inst.components.spell.spellname = "dango_light"
	inst.components.spell:SetVariables(light_variables)
	inst.components.spell.duration = TUNING.DANGO_DURATION
	inst.components.spell.ontargetfn = light_ontarget
	inst.components.spell.onstartfn = light_start
	inst.components.spell.onfinishfn = light_onfinish
	inst.components.spell.fn = light_spellfn
	inst.components.spell.resumefn = light_resume
	inst.components.spell.removeonfinish = true
	
	return inst
end

return Prefab("common/inventory/wharang_dango", fn, Assets),
Prefab("common/inventory/dango_light", lightfn)